export default class Fragment<TYPE extends string = string, T = any> {
    type: TYPE;
    data: T;
    constructor(type: TYPE, data: T);
}
export declare type Union = Fragment<'union', {
    type: Fragment<any>;
    type2: Fragment<any>;
}>;
export declare type Intersection = Fragment<'intersection', {
    type: Fragment<any>;
    type2: Fragment<any>;
}>;
export declare type Nullable = Fragment<'nullable', Fragment<any>>;
export declare type Tuple = Fragment<'tuple', Fragment<any>[] | null>;
export declare type ArrayFragment = Fragment<'array', {
    size: number;
    type: Fragment<any>;
}>;
export declare type ListFragment = Fragment<'list', Fragment<any>>;
export declare type Struct = Fragment<'struct', {
    [key: string]: {
        optional?: boolean;
        type: Fragment<any>;
    };
} | null>;
export declare type MapFragment = Fragment<'map', {
    keyType: Fragment<any>;
    type: Fragment<any>;
}>;
export declare type Named = Fragment<'named', {
    name: string;
    type: Fragment<any>;
}>;
export declare type Literal = Fragment<'literal', any>;
export declare type Primitive = Fragment<'primitive', string>;
export declare type Generic = Fragment<'generic', {
    extends?: Fragment<any>;
    name: string;
}>;
export declare type Any = Union | Intersection | Nullable | Tuple | ArrayFragment | ListFragment | Struct | MapFragment | Named | Literal | Primitive | Generic;
export interface Token {
    value: any;
    [key: string]: any;
}
export interface Lexer {
    reset: (chunk: string, info: any) => void;
    next: () => Token | undefined;
    save: () => any;
    formatError: (token: Token) => string;
    has: (tokenType: string) => boolean;
}
export interface NearleyRule {
    name: string;
    symbols: NearleySymbol[];
    postprocess?: (d: any[], loc?: number, reject?: {}) => any;
}
export declare type NearleySymbol = string | {
    literal: any;
} | {
    test: (token: any) => boolean;
};
export declare var Lexer: Lexer | undefined;
export declare var ParserRules: NearleyRule[];
export declare var ParserStart: string;
