"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var grammar_1 = require("./grammar");
var match_1 = require("./match");
var infer = function (value) {
    if (value === undefined)
        return new grammar_1.default('primitive', 'void');
    if (value === null)
        return new grammar_1.default('literal', null);
    switch (typeof value) {
        case 'boolean':
            return new grammar_1.default('primitive', 'bool');
        case 'number':
            return new grammar_1.default('primitive', 'float');
        case 'string':
            return new grammar_1.default('primitive', 'string');
        case 'object':
            if (Array.isArray(value)) {
                var length_1 = value.length;
                if (!length_1)
                    return new grammar_1.default('tuple', null);
                var types = value.map(infer);
                var firstType_1 = types[0];
                if (~types.findIndex(function (type) {
                    return !(type.type === 'primitive' && type.data === 'void' || match_1.default(firstType_1, type));
                })) {
                    return new grammar_1.default('tuple', types);
                }
                return new grammar_1.default('array', { type: firstType_1, size: length_1 });
            }
            var keys = Object.keys(value);
            if (!keys.length)
                return new grammar_1.default('struct', null);
            var object_1 = {};
            keys.forEach(function (key) {
                object_1[key] = { type: infer(value[key]) };
            });
            return new grammar_1.default('struct', object_1);
    }
    return new grammar_1.default('primitive', 'any');
};
exports.default = infer;
//# sourceMappingURL=infer.js.map